
#include "cprolog.h"
#include <tchar.h>   
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <stdarg.h>
#include <math.h>
#include <errno.h>
#include "tine.h"
#ifndef NO_REMOTE_PIPE
#include "shmalloc.h"
#endif
#ifdef MULTITHREADED
#include "threader.h"
#endif

extern int gSelectOnSysPoll;
unsigned int idCycleTimer = 0;
int cycleTerminated = FALSE;
int cycle_busy = 0;
int gSystemDelay = 20;
char gTineCycleWindowName[256];

HANDLE hTineCycleWnd = 0;
#define hNULL 0
void DoEvents(int maxevents);
int hasMessages(void);


void srvTmrTask(void)
{
  if (cycleTerminated) return;
  if (cycle_busy++) return;
  SystemCycle(FALSE);
  cycle_busy = 0;
}

#ifdef MULTITHREADED
DWORD dwTmrThreadId;
HANDLE hTmrThread = 0;
THREAD_TYPE tmrTaskThread(THREAD_PARAM lpvThreadParam)
{
  feclog("timer thread starting");
  while (!cycleTerminated)
  {
    srvTmrTask();
    // SystemCycle() should block on select with timeout of gSystemDelay
    if (cycle_busy) millisleep(gSystemDelay);
    
    if (hasMessages()) DoEvents(10);
      //millisleep(gSystemDelay);
  }
  return 0;
}
HANDLE tmrCreateThread(void)
{
  HANDLE hThread = 0;
  int threadPriority = -1;
  char *ptr = getenv("TINE_CYCLE_THREAD_PRIORITY");

  hThread = CreateThread(NULL,0,tmrTaskThread,0,0,&dwTmrThreadId);
  if (ptr != NULL) threadPriority = atoi(ptr);
  if (threadPriority > 0) 
  {
    feclog("cycle thread priority set to %d via environment",threadPriority);
    SetThreadPriority(hThread,threadPriority);
  }
  return hThread;
}
int isUsingWindowsCycleTimer = FALSE;
#else
int isUsingWindowsCycleTimer = TRUE;
#endif


__declspec(dllexport) LRESULT CALLBACK TineCyleWndProc(HWND hWnd, WORD iMessage, WORD wParam, LONG lParam)
{
  switch (iMessage)
  {
    case WM_TIMER:       
      srvTmrTask(); 
      if (!gSelectOnSysPoll ||   // select fell through
          ReturnToClientCycle || // client loop needs attention
          ReturnToServerCycle)   // server loop needs attention
        PostMessage(hTineCycleWnd,WM_TIMER,0,0);
      break;
    case WM_DESTROY:     
      KillTimer(hWnd,1);
    default: 
      return DefWindowProc (hWnd, iMessage, wParam, lParam);
  }
  return 0;
}

#if !defined(NO_REMOTE_PIPE)
int CreateTineCycleWindow(void)
{
  static WNDCLASS wndclass;
  static char  lpszWN[256];

  strncpy(lpszWN,gTineCycleWindowName,256); lpszWN[256] = 0;
  wndclass.style         = CS_GLOBALCLASS|CS_SAVEBITS;  // essential style!!
  wndclass.lpfnWndProc   = (WNDPROC) TineCyleWndProc;
  wndclass.cbClsExtra    = 0;
  wndclass.cbWndExtra    = 0;
  wndclass.hInstance     = hParent;
  wndclass.hIcon         = LoadIcon (hNULL, IDI_APPLICATION);
  wndclass.hCursor       = LoadCursor (hNULL, IDC_ARROW);
  wndclass.hbrBackground = GetStockObject (WHITE_BRUSH);
  wndclass.lpszMenuName  = hNULL;
  wndclass.lpszClassName = lpszWN;

  if (!RegisterClass(&wndclass)) return -1;

  if (!(hTineCycleWnd = CreateWindow (
			lpszWN, lpszWN,               // lpClassName, lpWindowName
			WS_BORDER|WS_CAPTION,         // dwStyle
			CW_USEDEFAULT, CW_USEDEFAULT, // X, Y
			CW_USEDEFAULT, CW_USEDEFAULT, // nWidth, nHeight
			hNULL, hNULL,                 // hWndParent, hMenu
			hParent,                      // hInstance
			(LPSTR)""                     // createstruct
      ) ) ) return -1;

  feclog("created timer Message Loop");
  return 0;
}
#endif

VOID CALLBACK srvTimerProc(
  HWND hwnd,         // handle to window
  UINT uMsg,         // WM_TIMER message
  UINT_PTR idEvent,  // timer identifier
  DWORD dwTime       // current system time
)
{
  srvTmrTask();
}

TINE_EXPORT void SystemSetCycleTimer(void)
{
  static int started = FALSE;
  if (started) return;
  started = TRUE;
  if (isUsingWindowsCycleTimer)
  {
	#if !defined(NO_REMOTE_PIPE)
    CreateTineCycleWindow();
	#endif
    idCycleTimer = SetTimer(hTineCycleWnd,1,gSystemDelay,NULL);
    //idCycleTimer = SetTimer(NULL,0,gSystemDelay,(TIMERPROC)srvTimerProc);
    if (idCycleTimer == 0) 
    {
      feclog("failed to create cycle timer : %d",GetLastError());
    }
  }
# ifdef MULTITHREADED
  else
  {
    UseMultiThreadedCycler(TRUE);
    hTmrThread = tmrCreateThread();
    millisleep(gSystemDelay);
  }
# endif
}
TINE_EXPORT void SystemKillCycleTimer(void)
{
  if (cycleTerminated) return;
  cycleTerminated = TRUE;
# ifdef MULTITHREADED
  if (!isUsingWindowsCycleTimer)
  {
    millisleep(gSystemDelay*2);
  }
# endif
}
TINE_EXPORT int SetSystemDelay(int msecs)
{
  if (msecs >= 10) gSystemDelay = msecs;
  if (gSystemDelay < MaxPollingRate)
  {
    MaxPollingRate = MinPollingRate = gSystemDelay;
  }
  return gSystemDelay;
}
int hasMessages(void)
{
  DWORD dw = GetQueueStatus(QS_SENDMESSAGE|QS_POSTMESSAGE);
  return dw == 0 ? FALSE : TRUE;
}
void DoEvents(int maxevents)
{
  MSG msg;
  int i = 0;
  while (PeekMessage(&msg,NULL,0,0,PM_REMOVE))
  {
    TranslateMessage(&msg);
    DispatchMessage(&msg);
    if (maxevents > 0 && i++ > maxevents) break;
  }
}



BOOL WINAPI DllMain(HINSTANCE hInstDll,DWORD fdwReason,LPVOID lpvReserved)
{
# ifdef MULTITHREADED
    char *ptr = getenv("TINE_USE_TIMER");
# endif
  struct _timeb tb;
  
#ifndef UNDER_CE
  _ftime(&tb);
#else
  GetSystemTime(&tb);
#endif
  sprintf(gTineCycleWindowName,"TineCycle @ %d:%d",tb.time,tb.millitm);  
# ifdef MULTITHREADED
  if (ptr != NULL)
  { // if not set use the default (currently set to TRUE)
    if (!stricmp(ptr,"YES") || !stricmp(ptr,"TRUE")) isUsingWindowsCycleTimer = TRUE;
    if (!stricmp(ptr,"NO") || !stricmp(ptr,"FALSE")) isUsingWindowsCycleTimer = FALSE;
  }
# endif
#if !defined(NO_REMOTE_PIPE)
  switch (fdwReason)
  {
    case DLL_PROCESS_ATTACH:  // DLL being loaded
      hParent = hInstDll;
      break;
    case DLL_THREAD_ATTACH:
      break;
    case DLL_THREAD_DETACH:
      break;
    case DLL_PROCESS_DETACH:  // DLL being unloaded
      if (hTineCycleWnd != NULL)
      {
        DestroyWindow (hTineCycleWnd);
        UnregisterClass(gTineCycleWindowName,hParent);
      }
      DestroyShmWindow();
      break;
  }
#endif
  return (TRUE);
  
}
